
#   Copyright (C) 2006 David G. Jacobowitz 
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#

package csv_funcs;
use Exporter 'import';
@EXPORT_OK = qw(read_csv read_csv_old write_csv print_csv 
                write_csv_dref);
use filewrap qw(open_read);


sub split_line_tdf {

 $il = shift;
 my $fragment;
 my $curr_elem;
 my $quote = 0;
 my $resref = ();
 my @p1 = split(/\t/,$il);
 foreach $fragment (@p1) {
  if      ($fragment =~ /^".*"$/) {
   $curr_elem = $fragment;
  } elsif ($fragment =~ /^"/) {
   $quote = 1;
   $curr_elem = $fragment;
  } elsif ($fragment =~ /"$/) {
   $quote = 0;
   $curr_elem .= "\t" . $fragment;
  } else {
   if ($quote) {
    $curr_elem .= "\t" . $fragment;
   } else {
    $curr_elem = $fragment;
   }
  }

  if (!$quote) {
   $curr_elem =~ s/^"//;
   $curr_elem =~ s/"$//;
   if ($curr_elem eq "") {
    $curr_elem = "__undef";
   }
   push(@{$resref}, $curr_elem);
  }
 }
 return $resref;
}


sub split_line_csv {

 $il = shift;
 my $fragment;
 my $curr_elem;
 my $quote = 0;
 my $resref = ();
 my @p1 = split(/,/,$il);
 foreach $fragment (@p1) {
  if      ($fragment =~ /^".*"$/) {
   $curr_elem = $fragment;
  } elsif ($fragment =~ /^"/) {
   $quote = 1;
   $curr_elem = $fragment;
  } elsif ($fragment =~ /"$/) {
   $quote = 0;
   $curr_elem .= "," . $fragment;
  } else {
   if ($quote) {
    $curr_elem .= "," . $fragment;
   } else {
    $curr_elem = $fragment;
   }
  }

  if (!$quote) {
   $curr_elem =~ s/^"//;
   $curr_elem =~ s/"$//;
   if ($curr_elem eq "") {
    $curr_elem = "__undef";
   }
   push(@{$resref}, $curr_elem);
  }
 }
 return $resref;
}


sub read_csv {
 my $infile = shift;
 my $tdf    = shift;

 if (!defined($tdf)) { $tdf = 0; };
 my $aa = [];
 my $names = [];
 my $row = -1;
 my $col = 0;
 my ($fok,$fh) = open_read($infile);
 if ($fok) {
  while (<$fh>) {
   my $il = $_;
   chomp $il;
   # print "IL = \"$il\"\n";
   # for (my $i=0;$i<length($il);$i++) {
   #  printf("%2.2x %c\n", ord(substr($il,$i,1)),ord(substr($il,$i,1)));
   # }

   if ($il =~ /^\*/) {
   } else {
    my $oneline_ref;
    if ($tdf) {
     $oneline_ref = split_line_tdf($il);
    } else {
     $oneline_ref = split_line_csv($il);
    } 
    if ($row == -1) {
     for ($col=0;$col<@{$oneline_ref};$col++) {
      $names->[$col] = $oneline_ref->[$col];
     }
    } else {
     for ($col=0;$col<@{$names};$col++) {
      $aa->[$col][$row] = defined($oneline_ref->[$col]) ?
                                  $oneline_ref->[$col] : "__undef";
     }
    }
    $row++;
   }
  }
  close $fh;
 }
 return ($names,$aa);
}


sub read_csv_old {
 my $infile = shift;
 my $tdf    = shift;

 if (!defined($tdf)) { $tdf = 0; };
 my $ary_ary_ref = ();
 my ($fok,$fh) = open_read($infile);
 if ($fok) {
  while (<$fh>) {
   my $il = $_;
   chomp $il;
   if ($il =~ /^\*/) {
   } else {
    my $oneline_ref;
    if ($tdf) {
     $oneline_ref = split_line_tdf($il);
    } else {
     $oneline_ref = split_line_csv($il);
    } 
    push(@{$ary_ary_ref}, $oneline_ref); 
   }
  }
  close $fh;
 }
 return $ary_ary_ref;
}


sub print_csv {
 my $aa_ref  = shift;
 my $drop_na = shift;
 if (!defined($drop_na)) { $drop_na = 0; };
 my $ol = "";
 foreach my $y (@{$aa_ref}) {
  foreach my $x (@{$y}) {
   if      ($x eq "") {
   } elsif ($drop_na && (lc($x) =~ /n[\/]+a/)) {
    $ol .= ".";
   } elsif (!defined($x)) {
   } elsif ($x =~ /^\d+$/) {
    $ol .= $x
   } elsif ($x =~ /^\d+\.\d*$/) {
    $ol .= $x;
   } else {
    $ol .= "\"" . $x . "\"";
   }
  $ol .= ",";
  }
 chop $ol;
 $ol .= "\n";
 }
 return $ol;
}

sub write_csv {
 my $aa_ref  = shift;
 my $ofile   = shift;
 my $drop_na = shift;
 local *OF;
 open (OF, "> " . $ofile) or die "Could not open file $ofile for write.\n";
 my $ol = print_csv($aa_ref, $drop_na);
 print OF $ol;
 close OF;
}

sub write_csv_dref {
 my $dref   = shift;
 my $ofile  = shift;
 my $fields = shift;
 open (OF, "> " . $ofile) or die "Could not open file $ofile for write.\n";

 #first, print the variable names, if we've got 'em
 my $ol ="";
 foreach my $item (@{$fields}) {
  $ol .= $item . ",";	  
 }
 chop $ol;
 print OF $ol . "\n";

 # now print the data:
 foreach my $entry (sort keys %{$dref}) {
  my $ol = "";
  my $nogood = 0;
  foreach my $item (@{$fields}) {
   if (defined($dref->{$entry}{$item})) {
    $ol .= $dref->{$entry}{$item} . ","; 
   } else {
    print "-Warning- column $item not found in row $entry. Will skip.\n";
    $nogood = 1;
   }
  }
  if (!$nogood) {
   chop $ol;
   print OF $ol . "\n";
  }
 }
 close OF; 
}




1;

