#!/usr/bin/perl -w

#   Copyright (C) 2006 David G. Jacobowitz 
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#

use fgroup;
package fgroup::corr;
BEGIN { @ISA = qw(fgroup); };

use Exporter 'import';
@EXPORT_OK = qw();

use Data::Dumper;
use Scalar::Util qw(looks_like_number);
use misc qw(covariance fixedFloat fixedFloat_p deep_copy);
use misc2 qw(func_fail);
use dataqual qw(gen_numquald_ha);

# use Statistics::Distributions;
use mod_distributions;

use strict;

sub commands {
 my $self = shift;
 my @cmds = ("corr");
 return @cmds;
}

sub corr {
 my $self = shift;
 my $a    = shift;
 my $oo   = $self->{oo};

 my $r = {};
 $r->{status} = "ok";

 my @vars = ();
 foreach my $v (@{$a->{varary}}) {
  if (defined($v->{state}) &&
      ($v->{state} eq "exist") &&
      ($self->{dm}{info}{$v->{name}}{type} eq "numeric")) {
   push(@vars, $v->{name});
  } else {
   if (defined($v->{name})) {
    $r->{status} = "fail; variable $v->{name} not declared or not numeric";
   } else {
    $r->{status} = "fail; no variables supplied";
   }
   return $r;
  }
 }

 if (@vars < 2) {
  $r->{status} = "fail; fewer than two variables specified";
  return $r;
 }

 my $dha = gen_numquald_ha($self->{dm},\@vars,$a->{ifary});

 my $obscount = @{$dha->{data}{$vars[0]}};

 my @vrefs = ();
 my $results = {};
 foreach my $vy (@vars) {
  foreach my $vx (@vars) {
   if ($vy eq $vx) {
     $results->{$vy}{$vx} = 1;
   } elsif (defined($results->{$vx}{$vy})) {
     $results->{$vy}{$vx} = $results->{$vx}{$vy};
   } else {
    my $tr = covariance($dha->{data}{$vx},$dha->{data}{$vy});
    $results->{$vy}{$vx} = $tr->{correlation}; 
    if (func_fail($tr)) {
     $r->{status} = "ok; but some correlations failed because of zero std. dev.";
    }
   }
  }
 }

 my $os = sprintf("\n(obs=%d)\n\n",$obscount);
 $os .= "           |";
 foreach my $vx (keys %{$results->{$vars[0]}}) {
  $os .= sprintf(" %10.10s |",$vx);
 }
 $os .= "\n-----------+-";
 foreach my $vx (keys %{$results->{$vars[0]}}) {
  $os .= sprintf("-----------+-");
 }
 $os .= "\n";
 foreach my $vy (keys %{$results}) {
  $os .= sprintf("%10.10s |",$vy);
  foreach my $vx (keys %{$results->{$vy}}) {
    if (looks_like_number($results->{$vy}{$vx})) {
     $os .= sprintf(" %s |",fixedFloat($results->{$vy}{$vx})); 
    } else {
     $os .= sprintf(" %10.10s |",$results->{$vy}{$vx}); 
    }
  }
  $os .= "\n";
 }
 $os .= "\n\n";
 $oo->disp($os);
 $r->{corr_m} = $results;
 return $r;
}


1;
