#!/usr/bin/perl -w

#   Copyright (C) 2006 David G. Jacobowitz 
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#

use fgroup;
package fgroup::debugging;
BEGIN { @ISA = qw(fgroup); };

use Exporter 'import';
@EXPORT_OK = qw();

use warnings 'all';
use strict;


use Devel::Cycle;
use Data::Dumper;
use Time::HiRes qw(gettimeofday);


sub commands {
 my $self = shift;
 my @cmds = ("compact", "elapsed", "perl", "pdisp");
 push(@cmds,"cyclesleuth");
 return @cmds;
}

sub cyclesleuth {
 my $self = shift;
 my $a    = shift;
 my $oo   = $self->{oo};
 $oo->disp("cyclesleuth\n");
 $oo->disp("  searching: a\n");
 find_cycle($a);
 $oo->disp("  searching: dm\n");
 find_cycle($self->{dm});
 $oo->disp("  searching: gb\n");
 find_cycle($self->{gb});
 $oo->disp("  searching: rval\n");
 find_cycle($self->{rval});
 $oo->disp("  searching: rval2\n");
 find_cycle($self->{rval2});
 $oo->disp("  searching: oo\n");
 find_cycle($self->{oo});

# example cycle...
# my $mom = { name=>"Reina" };
# my $me  = { name=>"David",
#	     mother => $mom };
# my $baby = { name=>"Isolde",
#              father=> $me };
#
# $mom->{grandson} = $baby;
#
# $oo->disp("  searching: mom\n");
# find_cycle($mom);

 $oo->disp("\n");
 return { status => "ok" };
}


# useful command to measure time of calculations
sub elapsed {
 my $self = shift;
 my $oo   = $self->{oo};
 my $gb   = $self->{gb};

 sub hms {
  my $i = shift;
  my $h = int($i/(60*60));
  my $m = int(($i-($h*60*60))/60);
  my $s = $i-($h*60*60)-($m*60);
  return ($h,$m,$s);
 }

 my $now = [gettimeofday];
 my $time_since_start = $now->[0]+($now->[1]/1e6) - $gb->{start_time}[0] -
                                                ($gb->{start_time}[1]/1e6);
 my $time_since_last  = $now->[0]+($now->[1]/1e6) - $gb->{last_time}[0] -
                                                ($gb->{last_time}[1]/1e6);
 $gb->{last_time} = $now;

 my $os = "\nElapsed time:\n";
 my ($h,$m,$s) = hms($time_since_start);
 $os .= sprintf("  Since Startup      : %2.2d:%2.2d:%2.5f\n",$h,$m,$s);
 ($h,$m,$s) = hms($time_since_last);
 $os .= sprintf("  Since Last Elapsed : %2.2d:%2.2d:%2.5f\n",$h,$m,$s);
 $os .= "\n";
 $oo->disp($os);
 return { status => "ok" };
}


sub compact {
 my $self    = shift;
 delete $self->{gb}{rval};
 delete $self->{gb}{rval2};
 delete $self->{last_run};
 return { status => "ok" };
}

sub perl {
 my $self = shift;
 my $a    = shift;
 my $oo = $self->{oo};

 $oo->disp("Perlline: $a->{perlline}\n");

 my $os = eval $a->{perlline};
 $oo->disp("Perl eval response:\n$os\n\n");
 my $r = {};
 $r->{perlresp} = $os;
 $r->{status}   = "ok";
 return $r;
}

sub pdisp {
 my $self = shift;
 my $a    = shift;
 my $oo   = $self->{oo};
 my $l    = $a->{perlline};
 my $estr = "\$oo->disp(Dumper $l);";
 eval $estr;
 my $r = {};
 $r->{status}   = "ok";
 return $r;
}

1;

