#!/usr/bin/perl -w

#   Copyright (C) 2006 David G. Jacobowitz 
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#

use fgroup;
package fgroup::dos;
BEGIN { @ISA = qw(fgroup); };

use Exporter 'import';
@EXPORT_OK = qw();

use warnings 'all';
use strict;

use Cwd;
use Scalar::Util qw(looks_like_number);
use Data::Dumper;
use misc;
use misc2 qw(tstFlg);

sub commands {
 my $self = shift;
 my @cmds = qw(dir ls cd date pwd shell fork banner type cat);
 return @cmds;
}

sub pwd {
 my $self = shift;
 my $a    = shift;
 my $oo   = $self->{oo};
 my $dir  = Cwd::cwd();
 my $os .= "> " . $dir ."\n\n";
 $oo->disp($os);
 return {status => "ok",
         pwd => $dir };
}

sub shell {
 my $self   = shift;
 my $a      = shift;
 my $oo     = $self->{oo};
 my $result = `@{$a->{args}}`;
 $oo->disp("shell results: \n");
 $oo->disp($result . "\n");
 return { results => $result,
	  status => "ok"};
}

sub fork {
 my $self   = shift;
 my $a      = shift;
 my $oo     = $self->{oo};

 if (0) {
  ## this is not working right now. Hard crash with reported error:
  ## "free to wrong pool." Clearly the spunout thread is not handling
  ## it's demise very nicely.
  my $newpid = fork();
  if ($newpid) {
   $oo->disp("Started new process; id is $newpid\n");
  } else {
   my $results = exec(@{$a->{args}});
   exit(0);
  }
 } else {
  my $results = system(@{$a->{args}});
 }
  return { status => "ok"};
}

sub dir {
 my $self = shift;
 my $a    = shift;
 my $oo   = $self->{oo};
 my $r    = {};
 my @dirs = keys %{$a->{vars}};

 $r->{status} =  "ok";
 if (@dirs) {
 } else {
  push(@dirs,Cwd::cwd());
 }
 foreach my $dir (@dirs) {
  $oo->disp("Directory: $dir\n\n");
  my $ok = opendir(DD,$dir);
  if (!$ok) {
   return { status => "fail; could not open directory $dir" };
  }
  $r->{directory} = $dir;
  $self->{gb}{lastdir}{directory} = $dir;

  my @fnames = readdir(DD);
  closedir(DD);
  for (my $ct=0;$ct<@fnames;$ct++) {
   my $fn = $fnames[$ct];
   $oo->disp("[$ct] $fn");
   if (-d "$dir/$fn") {
    $oo->disp("/");
   }
   $oo->disp("\n");
  }
  $r->{files} = \@fnames;
  $self->{gb}{lastdir}{files} = \@fnames;
 }
 $oo->disp("\n\n");
 return $r;
}

sub ls {
 my $self = shift;
 my $a    = shift;
 return $self->dir($a);
}


sub cd {
 my $self = shift;
 my $a    = shift;
 my $argr = $a->{args};
 my $oo   = $self->{oo};
 my $r    = { status => "ok" };

 if (!$a->{firstvar}{present}) {
  return { status => "fail; directory not specified" };
 }

 my $newdir = "";
 if (looks_like_number($a->{firstvar}{name})) {
  if (defined($self->{gb}{lastdir}{files}[$a->{firstvar}{name}])) {
   $newdir = $self->{gb}{lastdir}{files}[$a->{firstvar}{name}];
  }
 } else {
  $newdir = $a->{firstvar}{name};
 }
 $r->{olddir} = Cwd::cwd;
 my $ok = chdir $newdir;

 # after a cd the last dir array should no longer exist since 
 # all its references are relative paths anyway
 $self->{gb}{lastdir} = undef;

 if ($ok) {
  $r->{newdir} = Cwd::cwd;
  $oo->disp("Directory now " . $r->{newdir} . "\n\n");
  return $r;
 } else {
  return { status => "fail; could not change to $newdir" };
 }
}


sub date {
 my $self = shift;
 my $oo   = $self->{oo};
 my $os = localtime(time());
 $oo->disp("        " . $os . "\n");
 return { date => $os,
	  status => "ok" };
}

sub banner {
 my $self = shift;
 my $oo   = $self->{oo};
 my $os   = misc::banner($self->{gb});
 $oo->disp($os);
 return { banner => $os,
	  status => "ok"};

}

sub type {
 my $self = shift;
 my $a    = shift;
 my $oo   = $self->{oo};
 my $gb   = $self->{gb};

 my $head = tstFlg($a,"head");
 my $hc   = $gb->{head_lines};

 if (defined($a->{firstvar}) &&
     $a->{firstvar}{present}) {
  my $fn = $a->{firstvar}{name};
  if ( -e $fn ) {
   open(IF,"<" . $fn);
   my $lc   = 0; 
   my $os   = "";
   while (<IF>) {
    $os .= $_; $lc++;
    if ($head && ($lc > $hc)) { goto typedone; };
   }
typedone:
   close IF;
   $oo->disp("\n" . $os . "\n\n");
   return => { status => "ok", text => $os };
  } else {
   return { status => "fail; could not open file \"$fn\""};
  }
 } else {
  return { status => "fail; no file specified" };
 }
}

sub cat {
 my $self = shift;
 my $a    = shift;
 return $self->type($a);
}

1;

