#!/usr/bin/perl -w

#   Copyright (C) 2006 David G. Jacobowitz 
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#

use fgroup;
package fgroup::log2do;
BEGIN { @ISA = qw(fgroup); };

use Exporter 'import';
@EXPORT_OK = qw();

use strict;

sub commands {
 my $self = shift;
 my @cmds = ("log2do");
 return @cmds;
}

sub log2do {
 my $self = shift;
 my $a    = shift;
 my $oo   = $self->{oo};

 if (!$a->{have_using}) {
  return { status => "fail; missing \"using\" value" };
 }
 my $ifn  = $a->{using};

 if (!( -e $ifn)) {
  return { status => "fail; input file \"$ifn\" does not exist" }
 }

 my $ofn = $ifn;
 $ofn =~ s/\....$//;
 $ofn .= ".do";

 if (defined($a->{args}[2]) && length($a->{args}[2])) {
  $ofn = $a->{args}[2];
 } 

 open(IF,"< " . $ifn) || return { status => "fail; could not open \"$ifn\"" };
 open(OF,"> " . $ofn) || return { status => "fail; could not open \"$ofn\"" };
 
 $oo->disp("Converting log file \"$ifn\" to do file \"$ofn\"\n\n");
 my $last    = "";
 my $last_go = 0;
 while (<IF>) {
  my $il = $_; chomp $il;

  if ($last_go && !($il =~ /^>\s*fail;/)) {
   print OF $last . "\n";
  }

  if ($il =~ s/^\s*\.\s*//) {
    $last = $il;
    $last_go = 1;
  } else {
    $last_go = 0;
  }
 }
 close IF;
 close OF;
 return { status => "ok" };
}

1;
